package org.astakhova.data.impl;

import org.astakhova.data.*;

public class ConditionNode extends Node implements IConditionNode {	
	/**
	 * Creates a new CondNode
	 * @param x X-coordinate
	 * @param y Y-coordinate
	 */
	ConditionNode(int x, int y){
		super(x, y);
		addOutArrow(null);
		addOutArrow(null);
		setHeight(40);
		setWidth(70);
	}
	
	/**
	 * Connects the node with another if the condition is fulfilled
	 * @param node a node this one is connected to
	 * @return created arrow
	 */
	Arrow connectTrueBranch(Node node) {
		if (getOutArrow(0) == null && node.canBeBound()) {
			Arrow a = new Arrow(this, node);
			a.setLabel("Yes");
			setOutArrow(0, a);
			node.addInArrow(a);
			return a;
		}
		return null;
	}
	
	/**
	 * Connects the node with another if the condition isn't fulfilled
	 * @param node a node this one is connected to
	 * @return created arrow
	 */
	Arrow connectFalseBranch(Node node) {
		if (getOutArrow(1) == null && node.canBeBound()) {
			Arrow a = new Arrow(this, node);
			a.setLabel("No");
			setOutArrow(1, a);
			node.addInArrow(a);
			return a;
		}
		return null;
	}
	
	/**
	 * Checks if the node isn't connected by true branch arrow
	 * @return true if there is no arrow connected with the node
	 */
	public boolean canBindByTrueBranch() {
		return (getOutArrow(0) == null) ? true : false;
	}
	
	/**
	 * Checks if the node isn't connected by false branch arrow
	 * @return true if there is no arrow connected with the node
	 */
	public boolean canBindByFalseBranch() {
		return (getOutArrow(1) == null) ? true : false;
	}
	
	public boolean canBind() {
		return false;
	}
	
	/**
	 * Does nothing 'cause knows nothing 
	 * about which branch to connect by
	 */
	Arrow connect(Node node) {
		return null;
	}
}