package org.astakhova.reader;

import javax.xml.parsers.*;
import org.astakhova.data.*;
import org.w3c.dom.*;
import org.xml.sax.SAXException;
import java.io.File;
import java.io.IOException;

/**
 * XML-file reader of a BDE-diagram.
 * Doesn't perform a validation
 * @author Astakhova Julia A.
 */
public class FileReader {
	private IDiagram myDiagram;
	private Document myDocument;
	
	/**
	 * Creates a new reader for the file
	 * @param file XML-file to read
	 * @throws SAXException
	 * @throws ParserConfigurationException
	 * @throws IOException
	 */
	public FileReader(File file) throws SAXException, 
		ParserConfigurationException, IOException {
		myDocument = getBuilder().parse(file.getPath());
	}
	
	/**
	 * Reads a file into the diagram
	 * @param d diagram to form
	 */
	public void read(IDiagram d) {
		myDiagram = d;
		buildDiagram(myDocument.getDocumentElement());
	}
	
	private DocumentBuilder getBuilder() 
		throws SAXException, ParserConfigurationException {
		DocumentBuilderFactory factory = 
			DocumentBuilderFactory.newInstance();
		factory.setNamespaceAware(true);
		factory.setValidating(false);
		return factory.newDocumentBuilder();
	}
	
	private void buildDiagram(Element root) {
		Node n;
		NodeList nodes;
		
		nodes = root.getElementsByTagName("block");
		for (int i = 0; i < nodes.getLength(); i++) {
			n = nodes.item(i);
			addNode(n);
		}
		nodes = root.getElementsByTagName("arrow");
		for (int i = 0; i < nodes.getLength(); i++) {
			n = nodes.item(i);
			addArrow(n);
		}
	}
	
	private void addNode(Node n) {
		NamedNodeMap attrs = n.getAttributes();
		String type = attrs.getNamedItem("type").getNodeValue();
		int x = (new Integer(
				attrs.getNamedItem("x").getNodeValue())).intValue();
		int y = (new Integer(
				attrs.getNamedItem("y").getNodeValue())).intValue();
		int w = (new Integer(
				attrs.getNamedItem("width").getNodeValue())).intValue();
		int h = (new Integer(
				attrs.getNamedItem("height").getNodeValue())).intValue();
		INode cn = null;
		if (type.equals("data"))
			cn = myDiagram.createDataNode(x, y);
		if (type.equals("cond"))
			cn = myDiagram.createCondNode(x, y);
		if (type.equals("begin"))
			cn = myDiagram.createBeginNode(x, y);
		if (type.equals("end"))
			cn = myDiagram.createEndNode(x, y);
		if (cn == null)
			throw new RuntimeException(
					"ERROR : block of wrong type.\n");
		cn.setText(attrs.getNamedItem("text").getNodeValue());
		cn.setWidth(w);
		cn.setHeight(h);
	}
	
	private void addArrow(Node n) {
		NamedNodeMap attrs = n.getAttributes();
		int source = (new Integer(
			attrs.getNamedItem("source").getNodeValue())).intValue();
		int destination = (new Integer(
			attrs.getNamedItem("destination").getNodeValue())).intValue();
		Node pos = attrs.getNamedItem("branch");
		
		if (pos == null)
			myDiagram.createArrow(myDiagram.getNodes().get(source),
					myDiagram.getNodes().get(destination));
		else {
			boolean branch = 
				((new Integer(pos.getNodeValue())).intValue() == 0);
			String label = attrs.getNamedItem("label").getNodeValue();
			myDiagram.createArrow(
					(IConditionNode) myDiagram.getNodes().get(source),
					myDiagram.getNodes().get(destination), branch, label);
		}
	}
}
