package org.astakhova.view;

import java.awt.*;
import javax.swing.*;
import org.astakhova.data.IArrow;
import java.awt.font.*;
import org.astakhova.view.param.*;

/**
 * Makes a description of a coditional link
 * @author Astakhova Julia A.
 */
public class ConditionLink extends AbstractLink {
	private boolean myBranch;
	static final long serialVersionUID = 1;
	private JTextField myEditableComponent;
	
	/**
	 * Creates a new conditional link
	 * @param a an arrow
	 * @param branch branch of the link
	 */
	public ConditionLink(IArrow a, boolean branch) {
		super(a);
		myBranch = branch;
	}
	
	public void paintComponent(Graphics g) {		
		int srcLeft = getArrow().getSource().getX();
		int srcTop = getArrow().getSource().getY();
		int srcRight = srcLeft + getArrow().getSource().getWidth();
		int srcBottom = srcTop + getArrow().getSource().getHeight();
		 
		int tgtLeft = getArrow().getDest().getX();
		int tgtTop = getArrow().getDest().getY();
		int tgtRight = tgtLeft + getArrow().getDest().getWidth();
		int tgtBottom = tgtTop + getArrow().getDest().getHeight();
		
		g.setColor(getColor());
		
		Point sc = new Point(), b = new Point(), e = new Point();
		b.y = e.y = (srcBottom - srcTop) / 2 + srcTop;
		if (myBranch) {
			b.x = srcLeft;
			e.x = b.x - 20;
			sc.x = b.x - (int) getLabelLength((Graphics2D) g) + 3;
			sc.y = e.y - 4;
		} else {
			b.x = srcRight;
			e.x = b.x + 20;
			sc.x = b.x;
			sc.y = b.y - 4;
		}
		if (myEditableComponent != null) {
			myEditableComponent.setForeground(getColor());
			myEditableComponent.setBounds(new Rectangle(
					sc.x, sc.y - 15, 
					(int) getLabelLength((Graphics2D) g), 15));
			myEditableComponent.setHorizontalAlignment(
					myBranch ? JTextField.RIGHT : JTextField.LEFT);
			myEditableComponent.setOpaque(false);
			myEditableComponent.setBorder(null);
			myEditableComponent.setForeground(SColor.selecting());
		} else
			g.drawString(getArrow().getLabel(), sc.x, sc.y);
		g.drawLine(b.x, b.y, e.x, e.y);
		
		setParameters(e.x, e.x, e.y, e.y, 
				tgtLeft, tgtRight, tgtTop, tgtBottom);
		super.paintComponent(g);
	}
	
	public void setEditableComponent(JTextField tf) {
		myEditableComponent = tf;
	}
	
	public JTextField getEditableComponent() {
		return myEditableComponent;
	}
	
	/**
	 * Gets a length of the label of the link
	 * @param g2 graphics
	 * @return a length of the label of the link
	 */
	private float getLabelLength(Graphics2D g2) {
		FontRenderContext frc = g2.getFontRenderContext();
        return (float) g2.getFont().getStringBounds(getArrow().getLabel(), 
        		frc).getWidth();
	}
}
