package org.astakhova.view;

import com.sun.image.codec.jpeg.*;
import javax.swing.*;
import java.awt.FlowLayout;
import java.awt.image.*;
import java.awt.event.*;
import org.astakhova.data.*;
import org.astakhova.reader.FileReader;
import org.astakhova.view.listener.DeleteMouseL;
import org.astakhova.view.listener.MouseL;
import org.astakhova.view.listener.NewArrowMouseL;
import org.astakhova.view.listener.NewBeginBlockMouseL;
import org.astakhova.view.listener.NewBlockMouseL;
import org.astakhova.view.listener.PointerMouseL;
import org.astakhova.writer.FileWriter;
import java.io.*;
import javax.swing.filechooser.FileFilter;

/**
 * Makes a description of a main form
 * @author Astakhova Julia A.
 */
public class MainForm extends JFrame {
	private int myDiagramScore = 0;
	private static final long serialVersionUID = 1;
	private JDesktopPane myDesktop;
	private JToolBar myToolBar;
	private JFileChooser myFileChooser = new JFileChooser();
	private ButtonGroup myButtonGroup = new ButtonGroup();
	
	/**
	 * Creates a new main form
	 */
	public MainForm() {
		super("BlockDiagramEditor");
		setDefaultCloseOperation(DISPOSE_ON_CLOSE);
		setSize(700, 550);
		setLocation(140, 60);
		
		createToolBar();
		JMenuBar menubar = new JMenuBar();
		menubar.add(createFileMenu());
		menubar.add(createEditMenu());
		menubar.add(createHelpMenu());
		menubar.add(myToolBar);
		setJMenuBar(menubar);	
		
		myDesktop = new JDesktopPane();
		myDesktop.setDesktopManager(new DefaultDesktopManager() {
			static final long serialVersionUID = 1;
			public void activateFrame(JInternalFrame f) {
				super.activateFrame(f);
				DiagramForm df = (DiagramForm) f;
				bindButtons(df);
				checkButtonPossibility();
			}
			public void deactivateFrame(JInternalFrame f) {
				super.deactivateFrame(f);
				disableButtons();
				((DiagramForm) f).setUnselected();
			}
		});
		
		addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				closeWindow();
				super.windowClosing(e);
			}
		});
		
		myFileChooser.addChoosableFileFilter(new XMLFileFilter());
		setContentPane(myDesktop);
		setVisible(true);		
		createDiagram(DiagramFactory.newDiagram());
	}
	
	private boolean closeWindow() {
		boolean b = false;
		setDefaultCloseOperation(DISPOSE_ON_CLOSE);
		for (JInternalFrame f : myDesktop.getAllFrames()) {
			if (((DiagramForm) f).isChanged())
				b = true;
			((DiagramForm) f).setUnselected();
		}
		if (b) {
			int r = JOptionPane.showConfirmDialog(MainForm.this, 
					"Would you like to save changes?");
			if (r == JOptionPane.YES_OPTION) {
				for (JInternalFrame ff : myDesktop.getAllFrames()) {
					if (((DiagramForm) ff).isChanged())
						try {
							ff.setSelected(true);
							(new SaveAction()).actionPerformed(null);
						} catch (Exception q) {
							throw new 
								RuntimeException("Error while internal frame selecting");
						}
					
				}
			} else if (r == JOptionPane.CANCEL_OPTION) {
				setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
				return false;
			}
		}
		return true;
	}
	
	public static void main(String[] args) {
		new MainForm();
	}
	
	/**
	 * Gets an example of diagram
	 * @return example diagram
	 */
	/*private static IDiagram getDiagram() {
		IDiagram d = DiagramFactory.newDiagram();
		
		INode n1 = d.createDataNode(150, 50);
		n1.setText("B");
		INode n4 = d.createDataNode(105, 200);
		n4.setText("DDD");
		INode n0 = d.createDataNode(200, 200);
		n0.setText("AAAAAAAAAA");
		d.createEndNode(150, 300);
		INode n5 = d.createCondNode(145, 120);
		n5.setText("F");
		d.createBeginNode(150, 10);
		
		d.createArrow(n4, n0);
		d.createArrow(d.getNode(150, 10), n1);
		d.createArrow(n1, n5);
		d.createArrow((IConditionNode) n5, n4, true);
		d.createArrow((IConditionNode)n5, n0, false);
		d.createArrow(n0, d.getNode(150, 300));
			
		return d;
	}*/
	
	private JMenu createFileMenu() {
		JMenu file = new JMenu("File");
		ListenableMenuItem nw = new ListenableMenuItem(new NwAction());
		ListenableMenuItem open = new ListenableMenuItem(new OpenAction());
		ListenableMenuItem save = new ListenableMenuItem(new SaveAction());
		ListenableMenuItem saveas = new ListenableMenuItem(new SaveAsAction());
		ListenableMenuItem export = new ListenableMenuItem(new ExportAction());
		ListenableMenuItem exit = new ListenableMenuItem(new ExitAction());
		file.add(nw);
		file.add(open);
		file.addSeparator();
		file.add(save);
		file.add(saveas);
		file.addSeparator();
		file.add(export);
		file.addSeparator();
		file.add(exit);
		return file;
	}
	
	class ListenableMenuItem extends JMenuItem {
		static final long serialVersionUID = 1;
		
		public ListenableMenuItem(Action a) {
			super(a);
			this.addActionListener(new ActionListener() {
				
				// It's neccessary to unable selected items through active form
				// not to have problems with repainting text fields java components
				public void actionPerformed(ActionEvent e) {
					try {
						DiagramForm df = (DiagramForm) myDesktop.getSelectedFrame();
						df.setUnselected();
					} catch (Exception ee) {}
				}
			});
		}
	}
	
	class NwAction extends AbstractAction {
		private static final long serialVersionUID = 1;
		
		NwAction() {
			putValue(NAME, "New");
		}
		public void actionPerformed(ActionEvent e) {
			createDiagram(DiagramFactory.newDiagram());
		}
	}
	
	class OpenAction extends AbstractAction {
		private static final long serialVersionUID = 1;
		
		OpenAction() {
			putValue(NAME, "Open...");
		}
		public void actionPerformed(ActionEvent e) {
			myFileChooser.setDialogTitle("Open diagram");
			int res = myFileChooser.showOpenDialog(MainForm.this);
			if (res == JFileChooser.APPROVE_OPTION) {
				File f = myFileChooser.getSelectedFile();
				String name = f.getName();
				IDiagram d = readDiagram(f);
				createDiagram(d);
				if (d != null) {
					DiagramForm df = (DiagramForm) myDesktop.getSelectedFrame();
					int pista = name.indexOf(".");
					df.setTitle((pista < 0) ? name : name.substring(0, pista));
				}
			}
		}
	}
	
	class XMLFileFilter extends FileFilter {
		public boolean accept(File f) {
			if (f.isDirectory())
				return true;
			return (f.getName().endsWith("bde"));
		}
		
		public String getDescription() {
			return "BDE files (*.bde)";
		}
	}
	
	class SaveAction extends AbstractAction {
		private static final long serialVersionUID = 1;
		
		SaveAction() {
			putValue(NAME, "Save");
		}
		public void actionPerformed(ActionEvent e) {
			DiagramForm df = (DiagramForm) myDesktop.getSelectedFrame();
			if (df!= null) {
				if (df.getFile()!= null)
					writeDiagram(df.getDiagram(), df.getFile());
				else
					(new SaveAsAction()).actionPerformed(e);
				df.fixSaveChanges();
			} else 
				new ErrorForm(MainForm.this, null, 
						"No diagram was selected.", false);
		}
	}
	
	class SaveAsAction extends AbstractAction {
		private static final long serialVersionUID = 1;
		
		SaveAsAction() {
			putValue(NAME, "Save As...");
		}
		public void actionPerformed(ActionEvent e) {
			DiagramForm df = (DiagramForm) myDesktop.getSelectedFrame();
			if (df!= null) {
				myFileChooser.setDialogTitle("Save diagram");
				int res = myFileChooser.showSaveDialog(MainForm.this);
				if (res == JFileChooser.APPROVE_OPTION) {
					File f = myFileChooser.getSelectedFile();
					String name = f.getName(), path = f.getPath();
					if (writeDiagram(df.getDiagram(), path)) {
						df.setFile(path);
						int pista = name.indexOf(".");
						df.setTitle((pista < 0) ? name : name.substring(0, pista));
					}	
					df.fixSaveChanges();
				}
			} else
				new ErrorForm(MainForm.this, null, 
						"No diagram was selected.", false);
		}
	}
	
	class ExportAction extends AbstractAction {
		private static final long serialVersionUID = 1;
		
		ExportAction() {
			putValue(NAME, "Export...");
		}
		public void actionPerformed(ActionEvent e) {
			DiagramForm df = (DiagramForm) myDesktop.getSelectedFrame();
			
			if (df!= null) {
				JFileChooser fc = new JFileChooser();
				fc.addChoosableFileFilter(new FileFilter() {
					public boolean accept(File f) {
						if (f.isDirectory())
							return true;
						return (f.getName().endsWith("jpeg"));
					}
					
					public String getDescription() {
						return "Image files (*.jpeg)";
					}
				});
				
				fc.setDialogTitle("Export diagram");
				int res = fc.showSaveDialog(MainForm.this);
				if (res == JFileChooser.APPROVE_OPTION) {
					File f = fc.getSelectedFile();
					String path = f.getPath();
					exportDiagram(df, path);
				}
			} else
				new ErrorForm(MainForm.this, null, 
						"No diagram was selected.", false);
		}
	}
	
	private void exportDiagram(DiagramForm df, String path) {
		df.setUnBoxed();
		df.setCPBox(new CPBox(0, 0, df.getWidth(), df.getHeight()));
		df.selectCPBox();
		BufferedImage bi = df.getFrame();
		df.setUnBoxed();
		
		if ( bi == null)
			return;
		
		BufferedOutputStream out = null;
		try {
			out = new BufferedOutputStream(new 
					FileOutputStream(path));
			JPEGImageEncoder encoder = JPEGCodec.createJPEGEncoder(out);
		    JPEGEncodeParam param = encoder.getDefaultJPEGEncodeParam(bi);
		    int quality = 99;
			quality = Math.max(0, Math.min(quality, 100));
			param.setQuality((float)quality / 100.0f, false);
			encoder.setJPEGEncodeParam(param);
			encoder.encode(bi);
			out.close(); 
		} catch (Exception ee) {
			throw new RuntimeException("IOException while JPEG encoding");
		}
	}
	
	class ExitAction extends AbstractAction {
		private static final long serialVersionUID = 1;
		
		ExitAction() {
			putValue(NAME, "Exit");
		}
		public void actionPerformed(ActionEvent e) {
			if (closeWindow())
				dispose();
		}
	}
	
	private JMenu createEditMenu() {
		JMenu edit = new JMenu("Edit");
		edit.add(new JMenuItem(new CopyAction()));
		return edit;
	}
	
	class CopyAction extends AbstractAction {
		private static final long serialVersionUID = 1;
		
		CopyAction() {
			putValue(NAME, "Copy to clipboard");
		}
		public void actionPerformed(ActionEvent e) {
			DiagramForm df = (DiagramForm) myDesktop.getSelectedFrame();
			BufferedImage bi = df.getFrame();
			
			if ( bi == null)
				return;
			
			(new ClipboardImage()).copyToClipboard(bi, MainForm.this);
		}
	}
	
	private JMenu createHelpMenu() {
		JMenu help = new JMenu("Help");
		help.add(new JMenuItem(new AboutAction()));
		return help;
	}
	
	class AboutAction extends AbstractAction {
		private static final long serialVersionUID = 1;
		
		AboutAction() {
			putValue(NAME, "About...");
		}
		public void actionPerformed(ActionEvent e) {
			DiagramForm df = (DiagramForm) myDesktop.getSelectedFrame();
			df.setUnselected();
			JDialog dialog = createDialog("About...");
			dialog.setVisible(true);
		}
	}
	
	private JDialog createDialog(String title) {
		JDialog dialog = new JDialog(this, title, true);
		dialog.setDefaultCloseOperation(DISPOSE_ON_CLOSE);
		dialog.setLayout(new FlowLayout());
		JLabel label = new JLabel(
				"<html><center><h2>Block Diagram Editor 1.0</h2>" +
				"<p>Program for editing block schema diagrams." +
				"</p><p>Written by Julia Astakhova,</p><p>2006</p>" 
				+ "</center></html>");
		dialog.add(label);
		dialog.setBounds(250, 270, 400, 140);
		dialog.setResizable(false);
		return dialog;
	}
	
	private void createDiagram(IDiagram d) {
		if (d == null)
			return;
		DiagramForm df = new DiagramForm(this, 
				d, "Diagram " + (myDiagramScore + 1), myDiagramScore++);
		df.setVisible(true);
		myDesktop.add(df);
		try {
	        df.setSelected(true);
	    } catch (java.beans.PropertyVetoException e) {}
	}
	
	private IDiagram readDiagram(File file) {
		FileReader fr = null;
		IDiagram d = DiagramFactory.newDiagram();
		try {
			fr = new FileReader(file);
			fr.read(d);
		} catch (Exception e) {
			new ErrorForm(MainForm.this, e, 
						"Error while reading", false);
			return null;
		}
		return d;
	}
	
	private boolean writeDiagram(IDiagram d, String f) {
		try {
			FileWriter fw = new FileWriter(d);
			fw.write(f);
		} catch (Exception e) {
			new ErrorForm(MainForm.this, e, 
					"Error while writing.", false);
			return false;
		}
		return true;
	}
	
	public void createToolBar() {
		myToolBar = new JToolBar();
		new ToolButton("/org/astakhova/view/icon/pointer.gif", "Pointer", new PointerMouseL());
		new ToolButton("/org/astakhova/view/icon/data.gif", "DataNode", 
					new NewBlockMouseL(new DataBlockFactory()));
		new ToolButton("/org/astakhova/view/icon/cond.gif", "ConditionNode",
					new NewBlockMouseL(new ConditionBlockFactory()));
		new ToolButton("/org/astakhova/view/icon/begin.gif", "BeginNode", 
					new NewBeginBlockMouseL());
		new ToolButton("/org/astakhova/view/icon/end.gif", "EndNode", 
					new NewBlockMouseL(new EndBlockFactory()));
		new ToolButton("/org/astakhova/view/icon/link.gif", "Link", new NewArrowMouseL());
		new ToolButton("/org/astakhova/view/icon/delete.gif", "Delete", new DeleteMouseL());
	}
	
	public class ToolButton extends JToggleButton {
		static final long serialVersionUID = 1;
		private MouseL myListener;
		
		/**
		 * Creates a new button for the toolbar
		 * @param icon icon for the button
		 * @param tip labeltip for the button
		 * @param l mouselistener that is activated after pressing the button
		 */
		public ToolButton(String icon, String tip, MouseL l) {
			super(new ImageIcon(MainForm.class.getResource(icon)));
			addActionListener(getBlockButtonL());
			setToolTipText(tip);
			setEnabled(false);
			myListener = l;
			myButtonGroup.add(this);
			myToolBar.add(this);
		}
		
		public MouseL getListener() {
			return myListener;
		}
		
		public ActionListener getBlockButtonL() {
			return new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					DiagramForm d = (DiagramForm) myDesktop.getSelectedFrame();
					
					if (d != null) {
						d.setUnBoxed();
						d.removeMouseListener(d.getMouseL());
						d.removeMouseMotionListener(d.getMouseL());
						
						myListener.setForm(d);
						d.setMouseL(myListener);
					}
				}
			};
		}
	}
	
	/**
	 * Disables all buttons on the toolbar. It's used when no diagram form is opened
	 */
	public void disableButtons() {
		for (int i = 0; i < 7; i++)
			((JToggleButton) myToolBar.getComponent(i)).setEnabled(false);
	}
	
	/**
	 * Binds all buttons' mouse listeners to the active at the moment form
	 * @param df active at the moment form
	 */
	public void bindButtons(DiagramForm df) {
		ToolButton tb = null;
		for (java.awt.Component c : myToolBar.getComponents()) {
			if (c instanceof ToolButton) {
				MouseL l = ((ToolButton) c).getListener();
				l.setForm(df);
				
				if (((ToolButton) c).isSelected())
					tb = (ToolButton) c;
			}
		}
		
		if (tb != null) {
			df.removeMouseListener(df.getMouseL());
			df.removeMouseMotionListener(df.getMouseL());
			df.setMouseL(tb.getListener());
		}
	}
	
	/**
	 * Enables or disables toolbar buttons according to their action possibilities
	 */
	public void checkButtonPossibility() {
		for (java.awt.Component c : myToolBar.getComponents()) {
			if (c instanceof MainForm.ToolButton) {
				MainForm.ToolButton tb = (MainForm.ToolButton) c;
				tb.setEnabled(tb.getListener().isPossible());
			}
		}
	}
}