package org.astakhova.view;

import javax.swing.*;
import org.astakhova.view.param.*;
import java.awt.*;

/**
 * Makes a description of the block selection. Every moment only one block can have
 *  a selection through its diagram
 * @author Astakhova Julia A.
 */
public class Selection extends JComponent {
	static final long serialVersionUID = 1;
	private Block myBlock;
	private Color myColor = SColor.selecting();
	private final int myOffset = 3;
	private final int myLength = 5;
	private Point[] myPoints = new Point[4];
	
	/**
	 * Creates a selection for the block
	 * @param b a block
	 */
	public Selection(Block b) {
		myBlock = b;
		setSize(b.getHeight() * 5, b.getWidth() * 5);
		setCursor(new Cursor(Cursor.MOVE_CURSOR));
	}
	
	public void paintComponent(Graphics g) {
		super.paintComponent(g);
		
		g.setColor(myColor);
		myBlock.setSelected(myColor);
		
		for (int i = 0; i < 4; i++)
			myPoints[i] = new Point();
		myPoints[0].x = myBlock.getNode().getX() - myOffset;
		myPoints[0].y = myBlock.getNode().getY() - myOffset;
		myPoints[1].x = myPoints[0].x;
		myPoints[1].y = myBlock.getNode().getY() + 
			myBlock.getNode().getHeight() + myOffset;
		myPoints[2].x = myBlock.getNode().getX() + 
			myBlock.getNode().getWidth() + myOffset;
		myPoints[2].y = myPoints[0].y;
		myPoints[3].x = myPoints[2].x;
		myPoints[3].y = myPoints[1].y;
		
		g.drawLine(myPoints[0].x, myPoints[0].y, 
				myPoints[0].x + myLength, myPoints[0].y);
		g.drawLine(myPoints[0].x, myPoints[0].y, 
				myPoints[0].x, myPoints[0].y + myLength);
		
		g.drawLine(myPoints[1].x, myPoints[1].y, 
				myPoints[1].x + myLength, myPoints[1].y);
		g.drawLine(myPoints[1].x, myPoints[1].y, 
				myPoints[1].x, myPoints[1].y - myLength);
		
		g.drawLine(myPoints[2].x, myPoints[2].y, 
				myPoints[2].x - myLength, myPoints[2].y);
		g.drawLine(myPoints[2].x, myPoints[2].y, 
				myPoints[2].x, myPoints[2].y + myLength);
		
		g.drawLine(myPoints[3].x, myPoints[3].y, 
				myPoints[3].x - myLength, myPoints[3].y);
		g.drawLine(myPoints[3].x, myPoints[3].y, 
				myPoints[3].x, myPoints[3].y - myLength);
	}
	
	/**
	 * Disable selection
	 */
	public void unSelect() {
		myBlock.setSelected(SColor.usual());
	}
	
	/**
	 * Checks if the point is inside the selection border
	 * @param p a point
	 * @return true if the point is inside the selection border
	 */
	public boolean contains(Point p) {
		final int border = 5;
		return (((POffset.getX(p.x) >= myPoints[0].x - border
				&& POffset.getX(p.x) <= myPoints[0].x + 2 * border)
				|| (POffset.getX(p.x) <= myPoints[2].x + border
				&& POffset.getX(p.x) >= myPoints[2].x - 2 * border))
				&& ((POffset.getY(p.y) >= myPoints[0].y - border
				&& POffset.getY(p.y) <= myPoints[0].y + 2 * border)		
				|| (POffset.getY(p.y) <= myPoints[1].y + border
				&& POffset.getY(p.y) >= myPoints[1].y - 2 * border))
				);
	}
	
	public Block getBlock() {
		return myBlock;
	}
}
