package org.astakhova.view.listener;

import org.astakhova.view.AbstractLink;
import org.astakhova.view.Block;
import org.astakhova.view.ConditionBlock;
import org.astakhova.view.ConditionLine;
import org.astakhova.view.ConditionLink;
import org.astakhova.view.Line;
import org.astakhova.view.Link;
import org.astakhova.view.param.*;
import java.awt.event.MouseEvent;
import org.astakhova.data.*;
import java.util.*;

/***
 * Makes a description of mouse listener for "create link" action button
 * @author Astakhova Julia A.
 */
public class NewArrowMouseL extends MouseL {	
	public void mousePressed(MouseEvent e) {
		//creates a line
		if (getForm().getUnderCursor().getPart() == null) {
			if (checkBindPosNode()) 
				createLine(e);
		} else {
			//creates a conditional line
			if (getForm().getUnderCursor().getPart() == true) {
				if (checkBindPosCondNode(true))
					createLine(e); 
			} else
				if (checkBindPosCondNode(false))
					createLine(e);
		}
	}
	
	private void createLine(MouseEvent e) {
		List<Block> uclist = getForm().getUnderCursor().getBlockList();
		if (uclist.size() == 0)
			return;
		INode n = uclist.get(0).getNode();
		int srcLeft = n.getX();
		int srcTop = n.getY();
		int srcRight = srcLeft + n.getWidth();
		int srcBottom = srcTop + n.getHeight();
			
		Line l = (n instanceof IConditionNode) ? new ConditionLine() :
			new Line();
		l.setSelected(SColor.selecting());
			
		if (getForm().getUnderCursor().getPart() != null)
			((ConditionLine) l).setNode(n, getForm().getUnderCursor().getPart());
		else
			l.setNode(n);
			
		l.setParameters(
				srcLeft, srcRight, srcTop, srcBottom,
				POffset.getX(e.getX()), POffset.getX(e.getX()), 
				POffset.getY(e.getY()), POffset.getY(e.getY()));
		
		getForm().getContentPane().add(l);
		getForm().getUnderCursor().addLink(l);
		getForm().repaint();
	}
	
	private boolean checkBindPosNode() {
		List<Block> uclist = getForm().getUnderCursor().getBlockList();
		if (uclist.size() == 0)
			return false;
		return uclist.get(0).getNode().canBind();
	}
	
	private boolean checkBindPosCondNode(boolean branch) {
		List<Block> uclist = getForm().getUnderCursor().getBlockList();
		if (uclist.size() == 0)
			return false;
		if (!(uclist.get(0).getNode() 
				instanceof IConditionNode))
			return false;
		IConditionNode n = (IConditionNode) 
			uclist.get(0).getNode();
		return branch ? 
				n.canBindByTrueBranch() : n.canBindByFalseBranch();
	}
	
	public void mouseReleased(MouseEvent e) {
		List<Line> ucline = getForm().getUnderCursor().getLineList();
		if (ucline.size() == 0)
			return;
		
		List<Block> ucblock = getForm().getUnderCursor().getBlockList();
		if (ucblock.size() < 2 || ucblock.get(0) == ucblock.get(1)) {
			clearLine();
			return;
		}
		
		//convert a line into link
		AbstractLink al = null;
		Block dest = ucblock.get(1);
		if (getForm().getUnderCursor().getPart() == null) {
			al = new Link(getForm().getDiagram().createArrow(
					ucline.get(0).getNode(), 
					dest.getNode()));
		} else {
			if (getForm().getUnderCursor().getPart() == true) {
				al = new ConditionLink(getForm().getDiagram().createArrow(
						(IConditionNode) 
							ucline.get(0).getNode(), 
							dest.getNode(), true), 
						true);
			} else
				al = new ConditionLink(getForm().getDiagram().createArrow(
						(IConditionNode) 
							ucline.get(0).getNode(), 
							dest.getNode(), false), 
						false);
		}
		
		getForm().getOwner().checkButtonPossibility();
		getForm().fixChanges();
		getForm().getContentPane().add(al);
		clearLine();
		
		if (dest.getNode().canBind() && isPossible()) {
			getForm().getUnderCursor().addBlock(dest);
			dest.setSelected(SColor.selecting());
		}
	}
	
	private void clearLine() {
		getForm().getContentPane().remove(getForm().getUnderCursor().getLineList().get(0));
		getForm().getUnderCursor().clear();
		getForm().repaint();
	}

	public void mouseMoved(MouseEvent e) {
		//selecting a nodes a new link can be added to
		if (!isPossible())
			return;
		
		boolean wasSelected = false;
		getForm().getUnderCursor().clear();
		for (java.awt.Component b : getForm().getContentPane().getComponents()) {
			if (b instanceof ConditionBlock) {
				if (!wasSelected && ((Block) b).isInside(e.getPoint())) {
					if (((ConditionBlock) b).isInside(e.getPoint(), true) &&
							((IConditionNode)((Block) b).getNode()).canBindByTrueBranch()) {
						getForm().getUnderCursor().addCondBlock((ConditionBlock) b, true);
						wasSelected = true;
					}
					if (((ConditionBlock) b).isInside(e.getPoint(), false) &&
							((IConditionNode)((Block) b).getNode()).canBindByFalseBranch()) {
						getForm().getUnderCursor().addCondBlock((ConditionBlock) b, false);
						wasSelected = true;
					}
				} else
					((Block) b).setSelected(SColor.usual());
				continue;
			}
			if (b instanceof Block) {
				if (!wasSelected && ((Block) b).isInside(e.getPoint()) &&
						((Block) b).getNode().canBind()) {
					getForm().getUnderCursor().addBlock((Block) b);
					wasSelected = true;
				} else
					((Block) b).setSelected(SColor.usual());
			}
		}
			
		getForm().getUnderCursor().setSelected(SColor.selecting());
		getForm().repaint();
	}
	
	public void mouseDragged(MouseEvent e) {
		//selecting nodes a new link can be bind to
		List<Line> ucline = getForm().getUnderCursor().getLineList();
		if (ucline.size() > 0) {
			boolean wasSelected = false;
			
			List<Block> ucblock = getForm().getUnderCursor().getBlockList();
			if (ucblock.size() >= 2)
				getForm().getUnderCursor().removeBlock(ucblock.get(1));
			
			for (java.awt.Component b : getForm().getContentPane().getComponents()) {
				if (b instanceof Block) {
					Block dest = (Block) b;
					if (!wasSelected && dest.isInside(e.getPoint()) &&
						dest.getNode().canBeBound() && 
						!areAlreadyBound(ucblock.get(0).getNode(), dest.getNode())) {
							getForm().getUnderCursor().addBlock((Block) b);
							wasSelected = true;
					} else
						if ((Block) b != ucblock.get(0))
							((Block) b).setSelected(SColor.usual());
				}
			}
				
			getForm().getUnderCursor().setSelected(SColor.selecting());
			ucline.get(0).setTargetPoint(POffset.getX(e.getX()), POffset.getY(e.getY()));
			getForm().repaint();
			return;
		}
	}
	
	public boolean isPossible() {
		boolean can1 = false, can2 = false;
		INode b = null;
		for (INode n : getForm().getDiagram().getNodes())
			if (n.canBind() || (n instanceof IConditionNode &&
					(((IConditionNode)n).canBindByFalseBranch() ||
						((IConditionNode)n).canBindByTrueBranch()))) {
				can1 = true;
				b = n;
			}
		for (INode n : getForm().getDiagram().getNodes())
			if (n.canBeBound() && n != b && !areAlreadyBound(b, n))
				can2 = true;
		
		return (can1 && can2);
	}
	
	private boolean areAlreadyBound(INode source, INode dest) {
		for (IArrow a : dest.getOutArrows())
			if (a!= null && a.getDest() == source)
				return true;
		return false;
	}
}
