package org.astakhova.view.listener;

import org.astakhova.view.*;
import org.astakhova.view.param.*;
import java.awt.*;
import java.awt.event.MouseEvent;
import javax.swing.JTextField;
import org.astakhova.data.INode;

/**
 * Makes a description of mouse listener for pointer button
 * @author Astakhova Julia A.
 */
public class PointerMouseL extends MouseL {	
	public void mouseClicked(MouseEvent e) {
		getForm().setUnBoxed();
		if (getForm().getSelectedBlock() != null && 
				!getForm().getSelectedBlock().getBlock().isInside(e.getPoint()))
			getForm().setUnselected();
		
		if (e.getClickCount() == 2) {
			setSelected();
			return;
		}
		
		if ((getForm().getSelectedBlock() != null) && 
				getForm().getSelectedBlock().getBlock().isInside(e.getPoint())) {
			addEditableComponent();
		} 
		
		java.util.List<Line> ucline = getForm().getUnderCursor().getLineList();
		if (ucline.size() > 0 && ucline.get(0) instanceof ConditionLink) {
			ConditionLink cl = (ConditionLink) ucline.get(0);
			JTextField tf = new JTextField(cl.getArrow().getLabel());
			tf.addKeyListener(new LinkLabelEditL(getForm()));
			cl.setEditableComponent(tf);
			getForm().getContentPane().add(tf);
		}
	}
	
	private void setSelected() {
		java.util.List<Block> l = getForm().getUnderCursor().getBlockList();
		if (l.size() == 0)
			return;
		
		Block b = l.get(0);
		if (b!= null) {
			getForm().setUnselected();
			getForm().setSelectedBlock(new Selection(b));
			getForm().getContentPane().add(getForm().getSelectedBlock());
			addEditableComponent();
			
			getForm().repaint();
		} else
			getForm().setUnselected();
	}	
	
	private void addEditableComponent() {
		if (!(getForm().getSelectedBlock().getBlock() instanceof BeginEndBlock)) {
			JTextField tf = new JTextField(
					getForm().getSelectedBlock().getBlock().getNode().getText());
			tf.addKeyListener(new BlockLabelEditL(getForm()));
			tf.setHorizontalAlignment(JTextField.CENTER);
			getForm().getSelectedBlock().getBlock().setEditableComponent(tf);
			getForm().getContentPane().add(tf);
		}
	}
	
	public void mousePressed(MouseEvent e) {
		Selection s = getForm().getSelectedBlock();
		if (s == null || !s.contains(e.getPoint()) && !s.getBlock().isInside(e.getPoint()))
			getForm().setUnselected();
		
		if (getForm().getUnderCursor().getBlockList().size() == 0 && 
				getForm().getUnderCursor().getLineList().size() == 0) {
			getForm().setUnBoxed();
			getForm().setCPBox(new CPBox(POffset.getX(e.getX()), POffset.getY(e.getY()), 0, 0));
			getForm().getContentPane().add(getForm().getCPBox());
			getForm().repaint();
		}
	}
	
	public void mouseReleased(MouseEvent e) {
		if (getForm().getCPBox() != null && 
				getForm().getCPBox().getParameters().width == 0 &&
				getForm().getCPBox().getParameters().height == 0)
			getForm().setUnBoxed();
		getForm().setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
	}

	public void mouseMoved(MouseEvent e) {
		if (getForm().getCPBox() != null)
			return;
		
		boolean wasSelected = false;
		getForm().getUnderCursor().clear();
		for (java.awt.Component b : getForm().getContentPane().getComponents()) {
			if (b instanceof Block) {
				if (!wasSelected && ((Block) b).isInside(e.getPoint())) {
					getForm().getUnderCursor().addBlock((Block) b);
					wasSelected = true;
				} else
					((Block) b).setSelected(SColor.usual());
			}
			if (b instanceof AbstractLink) {
				if (!wasSelected && 
						((AbstractLink) b).isInside(e.getPoint())) {
					getForm().getUnderCursor().addLink((AbstractLink) b);
					wasSelected = true;
				} else
					((AbstractLink) b).setSelected(SColor.usual());
			}
		}
		
		Selection selection = getForm().getSelectedBlock();
		if (selection != null && selection.contains(e.getPoint()))
			getForm().setCursor(new Cursor(Cursor.MOVE_CURSOR));
		else
			getForm().setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
			
		getForm().getUnderCursor().setSelected(SColor.selecting());
		getForm().repaint();
	}
	
	private void changeSize(INode n, MouseEvent e) {
		int x = n.getX(), y = n.getY(), w = n.getWidth(), h = n.getHeight();
		int ex = POffset.getX(e.getX()), ey = POffset.getY(e.getY());

		if (Math.abs(ex - x) < Math.abs(x + w - ex)) {
			n.setX(ex);
			n.setWidth(Math.max(w + x - ex, 10));
		} else
			n.setWidth(Math.max(ex - x, 10));
		
		if (Math.abs(ey - y) < Math.abs(y + h - ey)) {
			n.setY(ey);
			n.setHeight(Math.max(h + y - ey, 10));
		} else
			n.setHeight(Math.max(ey - y, 10));
	}
	
	public void mouseDragged(MouseEvent e) {
		//changing block shape
		Selection s = getForm().getSelectedBlock();
		if (s!= null && getForm().getCursor().getType() == Cursor.MOVE_CURSOR) {
			INode n = s.getBlock().getNode();
			changeSize(n, e);
			getForm().fixChanges();
			getForm().repaint();
			return;	
		}
		
		//selecting a part of the diagram to copy to clipboard
		if (s == null && getForm().getCPBox() != null) {
			getForm().setCursor(new Cursor(Cursor.CROSSHAIR_CURSOR));
			Point p = getForm().getCPBox().getPoint();
			int x = Math.min(POffset.getX(e.getX()), p.x);
			int y = Math.min(POffset.getY(e.getY()), p.y);
			int w = Math.abs(POffset.getX(e.getX()) - p.x);
			int h = Math.abs(POffset.getY(e.getY()) - p.y);
			getForm().getCPBox().setParameters(x, y, w, h);
			getForm().selectCPBox();
			getForm().repaint();
			return;
		}
		
		//taking the block to a new place
		java.util.List<Block> uclist = getForm().getUnderCursor().getBlockList();
		if (s == null && uclist.size() > 0 && uclist.get(0).isInside(e.getPoint())) {
			getForm().setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
			INode b = uclist.get(0).getNode();
			b.setX(POffset.getX(e.getX()) - b.getWidth()/2);
			b.setY(POffset.getY(e.getY()) - b.getHeight()/2);
			getForm().fixChanges();
			getForm().repaint();
		}
	}
	
	public boolean isPossible() {
		return true;
	}
}