package org.astakhova.writer;

import org.astakhova.data.*;
import org.w3c.dom.*;
import javax.xml.parsers.*;
import java.io.*;
import javax.xml.transform.*;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import java.util.List;
import java.nio.charset.*;
import java.nio.*;

/**
 * XML-file writer for a BDE-diagram. Used encoding - UTF-8
 * @author Astakhova Julia A.
 */
public class FileWriter {
	private IDiagram myDiagram;
	private org.w3c.dom.Node myDocument;
	
	/**
	 * Creates a new file writer for the diagram
	 * @param diagram 
	 * @throws ParserConfigurationException
	 */
	public FileWriter(IDiagram diagram) throws 
		ParserConfigurationException {
		myDiagram = diagram;
		myDocument = createDocument();
	}
	
	private org.w3c.dom.Node createDocument() 
		throws ParserConfigurationException {
		Document doc = DocumentBuilderFactory.newInstance()
			.newDocumentBuilder().newDocument();
		Element root = doc.createElement("diagram");
		Element e;
		List<INode> list = myDiagram.getNodes();
		for (INode n : list) {
			e = doc.createElement("block");
			e.setAttribute("type", getType(n));
			e.setAttribute("x", (new Integer(n.getX())).toString());
			e.setAttribute("y", (new Integer(n.getY())).toString());
			e.setAttribute("text", toUTF(n.getText()));
			e.setAttribute("width", (new Integer(n.getWidth())).toString());
			e.setAttribute("height", (new Integer(n.getHeight())).toString());
			root.appendChild(e);
		}
		for (INode nn : list)
			for (IArrow a : nn.getOutArrows())
				if (a!= null) {
					e = doc.createElement("arrow");
					e.setAttribute("source", 
						(new Integer(list.indexOf(nn))).toString());
					e.setAttribute("destination", 
						(new Integer(list.indexOf(a.getDest()))).toString());
					if (nn instanceof IConditionNode) {
						e.setAttribute("branch",(new Integer(
								nn.getOutArrows().indexOf(a))).toString());
						e.setAttribute("label", toUTF(a.getLabel()));
					}
					root.appendChild(e);
				}
		return root;
	}
	
	public void write(String filename) throws IOException, 
		TransformerConfigurationException, TransformerException {
		java.io.FileWriter fw = new java.io.FileWriter(filename);
		Transformer tr = TransformerFactory.newInstance().newTransformer();
		
		tr.setOutputProperty(OutputKeys.INDENT, "yes");
		tr.setOutputProperty(OutputKeys.ENCODING, "UTF-8");
		
		tr.transform(new DOMSource(myDocument), new StreamResult(fw));
		fw.close();
	}
	
	private String getType(INode n) {
		if (n instanceof IDataNode)
			return "data";
		if (n instanceof IConditionNode)
			return "cond";
		if (n instanceof IBeginNode)
			return "begin";
		if (n instanceof IEndNode)
			return "end";
		return "";
	}
	
	//Converting the string to UTF
	private String toUTF(String original) {
		String s = "ERROR";
		try {
			//byte[] a = original.getBytes();
			byte[] b = original.getBytes(System.getProperty("file.encoding"));
			//String ss = new String(a, System.getProperty("file.encoding"));
			Charset cs1 = Charset.forName(System.getProperty("file.encoding"));
			CharBuffer cb1 = cs1.newDecoder().decode(ByteBuffer.wrap(b));
			Charset cs2 = Charset.forName("UTF8");
			ByteBuffer bb = cs2.newEncoder().encode(cb1);
			int offset = 0;
			for (byte asb : bb.array()) {
				if (asb == 0)
					offset++;
			}
			
			s = new String(bb.array());
			s = s.substring(0, s.length() - offset);
			b = null;
		
		
			
		} catch (Exception e) {
			throw new RuntimeException("Encoding error");
		}
		return s;
	}
}
